package org.fmi.ejb;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.lang.reflect.Method;

public class RemoteMethod implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 8489684346831214799L;
	
	private transient Method method;
	private transient Object[] arguments;
	
	private String beanName;
	
	public RemoteMethod(String beanName) {
		this.beanName = beanName;
	}

	public Object[] getArguments() {
		return arguments;
	}

	public void setArguments(Object[] arguments) {
		this.arguments = arguments;
	}

	public Method getMethod() {
		return method;
	}

	public void setMethod(Method method) {
		this.method = method;
	}

	public String getBeanName() {
		return beanName;
	}
	
	private void writeObject(ObjectOutputStream oos) throws IOException {
		oos.defaultWriteObject();
		oos.writeObject(method.getName());
		oos.writeObject(method.getDeclaringClass());
		oos.writeObject(method.getParameterTypes());
		oos.writeObject(arguments);
	}

	private void readObject(ObjectInputStream ois) 
		throws IOException,ClassNotFoundException {
		ois.defaultReadObject();
		if (isServer()) {
			EJBContainer contaner = EJBContainerFactory.getEJBContainer();
			ClassLoader current = Thread.currentThread().getContextClassLoader();
			try {
				Thread.currentThread().setContextClassLoader(
						contaner.getEJB(beanName).getClassLoader());
				String name = (String)ois.readObject();
				Class declaringClass = (Class)ois.readObject();
				Class[] parameters = (Class[])ois.readObject();
				try {
					method = declaringClass.getDeclaredMethod(name,parameters);
				} catch(Exception ex) {
					throw new IOException("Cannot find method " + name + " in class " + declaringClass);	
				}
				arguments = (Object[])ois.readObject();
			} finally {
				Thread.currentThread().setContextClassLoader(current);
			}
		} else {
			method = (Method)ois.readObject();
			arguments = (Object[])ois.readObject();
		}
		
	}
	
	private boolean isServer() {
		String server = System.getProperty("org.fmi.ejb.server");
		return server != null && server.equals("true");
	}
	
}
