package org.fmi.ejb;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.lang.reflect.Proxy;

public class SerializedClientProxy implements Serializable {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 6067486699868737036L;
	private RemoteInvocationHandler ih;
	private String beanName;
	private transient Class[] intfClasses;
	
	public SerializedClientProxy(RemoteInvocationHandler ih,
			String beanName,Class[] intf) {
		this.ih = ih;
		this.beanName = beanName;
		this.intfClasses = intf;
	}
	
	
	private void writeObject(ObjectOutputStream oos) throws IOException {
		oos.defaultWriteObject();
		oos.writeObject(intfClasses);
	}

	private void readObject(ObjectInputStream ois) 
		throws IOException,ClassNotFoundException {
		ois.defaultReadObject();
		if (isServer()) {
			EJBContainer contaner = EJBContainerFactory.getEJBContainer();
			ClassLoader current = Thread.currentThread().getContextClassLoader();
			try {
				Thread.currentThread().setContextClassLoader(
						contaner.getEJB(beanName).getClassLoader());
				intfClasses = (Class[])ois.readObject();
			} finally {
				Thread.currentThread().setContextClassLoader(current);
			}
		} else {
			intfClasses = (Class[])ois.readObject();
		}
	}
	
	public Object readResolve() {
		RemoteInvocationHandlerAdapter adapter = new RemoteInvocationHandlerAdapter(
				ih,beanName,intfClasses);
		ClassLoader loader = null;
		if (isServer()) {
			loader = EJBContainerFactory.getEJBContainer().getEJB(beanName).getClassLoader();
		} else {
			loader = intfClasses[0].getClassLoader();
		}
		return Proxy.newProxyInstance(loader,intfClasses,adapter);
	}
	
	private boolean isServer() {
		String server = System.getProperty("org.fmi.ejb.server");
		return server != null && server.equals("true");
	}
}
