package org.fmi.ejb.deployer;

import java.io.File;
import java.io.FileFilter;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.EJBException;

import org.fmi.ejb.EJBContainer;

public class Deployer implements Runnable {
	
	private File appDir;
	private EJBContainer container;
	private List<String> names;
	private final PrintStream ERR = System.err;
	private long lastModified;
	private boolean stop = false;
	
	public Deployer(File dir,EJBContainer container) {
		this.appDir = dir;
		this.container = container;
		this.names = new ArrayList<String>();
	}
	
	public void run() {
		lastModified = appDir.lastModified();
		checkAppDirContent();
		
		while (!stop) {
			if ( lastModified < appDir.lastModified() ) {
				lastModified = appDir.lastModified();
				checkAppDirContent();
			}
			try {
				Thread.sleep(2000);
			} catch(InterruptedException ex) {
				ex.printStackTrace(ERR);
			}
		}
	}
	
	private void checkAppDirContent() {
		File[] jars = appDir.listFiles(JarFilter.FILTER);
		for (File jar:jars) {
			if (names.contains(jar.getName())) {
				continue;
			}
			try {
				container.deploy(jar);
				names.add(jar.getName());
			} catch (EJBException ex) {
				ex.printStackTrace(ERR);
			}
		}
	}
	
	private static class JarFilter implements FileFilter {
		
		private JarFilter() {}
		
		private static final JarFilter FILTER = new JarFilter();
		
		public boolean accept(File pathname) {
			return pathname.isFile() && 
				pathname.getName().endsWith(".jar");
		}
		
	}

	public void stop() {
		stop = true;
	}
}
