package org.fmi.ejb.impl;

import java.lang.reflect.Method;

import javax.ejb.EJBException;

import org.fmi.ejb.ApplicationException;
import org.fmi.ejb.ChainsManager;
import org.fmi.ejb.ClientView;
import org.fmi.ejb.EJB;
import org.fmi.ejb.EJBInstance;
import org.fmi.ejb.EJBInterceptor;
import org.fmi.ejb.EJBInvocationContext;
import org.fmi.ejb.InstanceID;
import org.fmi.ejb.InvocationChain;


public abstract class AbstractEJBInvocationContext implements EJBInvocationContext {
	
	private ChainsManager manager;
	private InvocationChain chain;
	private Method intfMethod;
	private EJBInstance instance;
	private InstanceID instanceId;
	private int position;
	private Object[] arguments;
	private ClientView client;
	
	AbstractEJBInvocationContext(Method intfMethod,ChainsManager manager,InvocationChain chain,InstanceID instanceId) {
		this.manager = manager;
		this.chain = chain;
		this.instanceId = instanceId;
		this.intfMethod = intfMethod;
		this.position = 0;
	}
	
	protected abstract Object proceedFinal() throws ApplicationException, EJBException;  
	
	public Object proceed() throws ApplicationException, EJBException {
		if (isAtEnd()) {
			return proceedFinal();
		} else {
			try {
				EJBInterceptor interceptor = manager.getInteceptor(
					chain.interceptorAt(position));
				position++;
				return interceptor.proceed(this);
			} finally {
				position--;
			}
		}
	}

	protected final boolean isAtEnd() {
		return chain.size() == position;
	}
	
	public EJBInstance getEJBInstance() {
		return instance;
	}
	
	public void setEJBInstance(EJBInstance instance) {
		this.instance = instance;
	}

	public EJB getEJB() {
		return manager.getEJB();
	}
	
	public InstanceID getInstanceId() {
		return instanceId;
	}

	public InvocationChain getChain() {
		return chain;
	}

	public Method getIntfMethod() {
		return intfMethod;
	}

	public void setInstanceId(InstanceID instanceId) {
		this.instanceId = instanceId;
	}

	public Object[] getArguments() {
		return arguments;
	}

	public void setArguments(Object[] arguments) {
		this.arguments = arguments;
	}

	public ClientView getClient() {
		return client;
	}

	public void setClient(ClientView client) {
		this.client = client;
	}
}
