package org.fmi.ejb.impl;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ejb.EJBException;

import org.fmi.ejb.ApplicationException;
import org.fmi.ejb.CallbackInvocationChains;
import org.fmi.ejb.ChainsManager;
import org.fmi.ejb.ClientView;
import org.fmi.ejb.EJB;
import org.fmi.ejb.EJBInstance;
import org.fmi.ejb.EJBInterceptor;
import org.fmi.ejb.EJBInvocationContext;
import org.fmi.ejb.InstanceID;
import org.fmi.ejb.InvocationChain;
import org.fmi.ejb.InvocationChains;

public class DefaultChainsManager implements ChainsManager {
	
	private EJB ejb;
	private Map<Class,InvocationChains> chains;
	private List<EJBInterceptor> interceptors;
	private CallbackInvocationChains callbacks;
	
	DefaultChainsManager(EJB ejb) {
		this.ejb = ejb;
		this.chains = new HashMap<Class,InvocationChains>();
		interceptors = new ArrayList<EJBInterceptor>();
	}

	public Object startChain(InvocationChain chain, ClientView client, InstanceID id, Method method, Object[] args) throws ApplicationException, EJBException {
		RequestEJBInvocationContext ctx = new RequestEJBInvocationContext(method,this,chain,id);
		ctx.setClient(client);
		ctx.setArguments(args);
		return ctx.proceed();
	}

	public EJBInvocationContext startCallback(InvocationChain chain, InstanceID id) throws ApplicationException, EJBException {
		CallbackEJBInvocationContext ctx = new CallbackEJBInvocationContext(this,chain,id);
		return (EJBInvocationContext)ctx.proceed();
	}

	public EJBInvocationContext startCallback(InvocationChain chain, EJBInstance instance) throws ApplicationException, EJBException {
		CallbackEJBInvocationContext ctx = new CallbackEJBInvocationContext(this,chain,instance.getId());
		ctx.setEJBInstance(instance);
		return (EJBInvocationContext)ctx.proceed();
	}

	public EJBInterceptor getInteceptor(int id) {
		return interceptors.get(id);
	}

	public void register(InvocationChains chs) {
		this.chains.put(chs.getInterface(),chs);
		
	}

	public void register(CallbackInvocationChains chains) {
		this.callbacks = chains;
	}

	public int register(EJBInterceptor interceptor) {
		for (int i=0;i<interceptors.size();i++) {
			EJBInterceptor registered = interceptors.get(i);
			if (registered.equals(interceptor)) {
				return i;
			}
		}
		interceptors.add(interceptor);
		return interceptors.size()-1;
	}

	public InvocationChains getChains(Class intf) {
		return chains.get(intf);
	}

	public CallbackInvocationChains getCallbackChains() {
		return callbacks;
	}

	public EJB getEJB() {
		return ejb;
	}
}
