package org.fmi.ejb.impl;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import javax.interceptor.Interceptors;

import org.fmi.ejb.ChainsManager;
import org.fmi.ejb.ClientViewsManager;
import org.fmi.ejb.EJB;
import org.fmi.ejb.InstanceID;
import org.fmi.ejb.LifecycleManager;

public class DefaultEJB implements EJB {
	
	private String name;
	private ClassLoader loader;
	private boolean isStateful;
	private Class<?> beanClass;
	private List<Class<?>> interceptorClasses;
	private ClientViewsManager clientViewsManager;
	private LifecycleManager lifecycleManager;
	private ChainsManager chainsManager;
	private InstanceID containerID;
	
	DefaultEJB(String name,Class beanClass,ClassLoader loader, boolean isStateful) {
		this.name = name;
		this.beanClass = beanClass;
		this.loader = loader;
		this.isStateful = isStateful;
		this.containerID = new StatelessInstanceID(name);
		initInterceptorClasses();
	}
	
	private void initInterceptorClasses() {
		interceptorClasses = new ArrayList<Class<?>>();
		addInterceptorsToList((Interceptors)beanClass.getAnnotation(Interceptors.class),interceptorClasses);
		Method[] methods = beanClass.getMethods();
		for (Method method:methods) {
			addInterceptorsToList((Interceptors)method.getAnnotation(Interceptors.class),interceptorClasses);
		}
	}
	
	private void addInterceptorsToList(Interceptors interceptors,List<Class<?>> list) {
		if (interceptors != null) {
			for (Class interceptor:interceptors.value()) {
				if (!list.contains(interceptor)) {
					list.add(interceptor);
				}
			}
		}
	}

	public List<Class<?>> getInterceptorClasses() {
		return interceptorClasses;
	}
	
	
	public Class<?> getBeanClass() {
		return beanClass;
	}
	
	public String getName() {
		return name;
	}

	public ClassLoader getClassLoader() {
		return loader;
	}

	public boolean isStateful() {
		return isStateful;
	}

	public ClientViewsManager getClientViewsManager() {
		return clientViewsManager;
	}

	public LifecycleManager getLifecycleManager() {
		return lifecycleManager;
	}

	public ChainsManager getChainsManager() {
		return chainsManager;
	}

	protected void setLifecycleManager(LifecycleManager lifecycleManager) {
		this.lifecycleManager = lifecycleManager;
	}
	
	protected void setClientViewsManager(ClientViewsManager clientViewsManager) {
		this.clientViewsManager = clientViewsManager;
	}
	
	
	protected void setChainsManager(ChainsManager chainsManager) {
		this.chainsManager = chainsManager;
	}

	public InstanceID getContainerId() {
		return containerID;
	}
	

}
