package org.fmi.ejb.impl;

import java.lang.reflect.Method;

import javax.ejb.EJBException;

import org.fmi.ejb.ApplicationException;
import org.fmi.ejb.ChainsManager;
import org.fmi.ejb.ClientView;
import org.fmi.ejb.ClientViewFactory;
import org.fmi.ejb.EJBProxyInvocationHandler;
import org.fmi.ejb.InstanceID;
import org.fmi.ejb.InvocationChain;

public class DefaultEJBProxyInvocationHandler implements EJBProxyInvocationHandler {

	private ClientViewFactory factory;
	private ClientView clientView;
	private InstanceID id;
	protected Class clazz;
	
	DefaultEJBProxyInvocationHandler(InstanceID id,ClientViewFactory factory) {
		this.id = id;
		this.factory = factory;
		try {
			clazz = factory.getEJB().getClassLoader().loadClass(factory.getInterface());
		} catch(ClassNotFoundException cnfex) {
			throw new EJBException(cnfex);
		}
	}
	
	public ClientView getClientView() {
		return clientView;
	}
	
	public InstanceID getInstanceId() {
		return id;
	}

	public ClientViewFactory getFactory() {
		return factory;
	}
	
	public void setClientView(ClientView clientView) {
		this.clientView = clientView;
	}

	public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
		if (method.getDeclaringClass() == Object.class) {
			return method.invoke(this,args);
		}
		
		if (method.getDeclaringClass() == ClientView.class) {
			if (method.getName().equals("ejb_getInstanceID")) {
				return id;
			}
			if (method.getName().equals("ejb_getInterface")) {
				return factory.getInterface();
			}
			if (method.getName().equals("ejb_destroy")) {
				factory.destroyInstance(clientView);
				return null;
			}
			throw new IllegalArgumentException("Unknown method " + method );
		}
		ChainsManager manager = factory.getEJB().getChainsManager();
		InvocationChain chain = manager.getChains(clazz).getChain(method);
		try {
			return manager.startChain(chain,clientView,id,method,args);
		} catch(ApplicationException ex) {
			throw ex.getCause();
		}
	}

}
