package org.fmi.ejb.impl;

import java.util.ArrayList;
import java.util.List;

import javax.ejb.EJBException;

import org.fmi.ejb.ApplicationException;
import org.fmi.ejb.Callback;
import org.fmi.ejb.EJB;
import org.fmi.ejb.EJBInstance;
import org.fmi.ejb.EJBInvocationContext;
import org.fmi.ejb.InvocationChain;
import org.fmi.ejb.LifecycleManager;

public class StatelessLifecycleManager implements LifecycleManager {
	
	private EJB ejb;
	private List<EJBInstance> pool;
	
	StatelessLifecycleManager(EJB ejb) {
		this.ejb = ejb;
		this.pool = new ArrayList<EJBInstance>(10);
	}
	
	public EJBInstance getReady() throws ApplicationException, EJBException {
		synchronized (pool) {
			if (pool.size() == 0) {
				InvocationChain chain = ejb.getChainsManager().getCallbackChains().
					getChain(Callback.NEW);
				EJBInvocationContext ctx = ejb.getChainsManager().
					startCallback(chain,ejb.getContainerId());
				return ctx.getEJBInstance();
			} 
			EJBInstance instance = pool.remove(pool.size()-1);
			return instance;
		}
	}

	public void releaseReady(EJBInstance instance) throws EJBException { 
		synchronized (pool) {
			pool.add(instance);
		}
	}
	
	public void discard(EJBInstance instance) throws ApplicationException, EJBException {
		InvocationChain chain = ejb.getChainsManager().getCallbackChains().
			getChain(Callback.DISCARD);
		ejb.getChainsManager().
			startCallback(chain,instance);
	}
	
	public void destroy(EJBInstance instance) throws ApplicationException, EJBException {
		InvocationChain chain = ejb.getChainsManager().getCallbackChains().
			getChain(Callback.DESTROY);
		ejb.getChainsManager().
			startCallback(chain,instance);
	}
	
	public EJB getEJB() {
		return ejb;
	}
	
}
